@@ -36,6 +36,8 @@
 #include <QtDBus/QDBusInterface>
 #include <QtDBus/QDBusConnection>
 #include <QtDBus/QDBusMessage>
+#include <QtDBus/QDBusServiceWatcher>
+#include <QItemSelectionModel>
 #include <QTimer>
 #include <QFileInfo>
 #include <QStringBuilder>
@@ -54,7 +56,7 @@ ColordKCM::ColordKCM(QWidget *parent, const QVariantList &args) :
     aboutData = new KAboutData("kcm_colord",
                                "kcm_colord",
                                ki18n("Color settings"),
-                               "0.1",
+                               "0.2",
                                ki18n("Color settings"),
                                KAboutData::License_GPL,
                                ki18n("(C) 2012 Daniel Nicoletti"));
@@ -92,42 +94,55 @@ ColordKCM::ColordKCM(QWidget *parent, const QVariantList &args) :
     // Connect this slot prior to defining the model
     // so we get a selection on the first item for free
     connect(sortModel, SIGNAL(dataChanged(QModelIndex,QModelIndex)),
-            this, SLOT(showProfile()));
+            this, SLOT(showDescription()));
     sortModel->setDynamicSortFilter(true);
     sortModel->setSortRole(DeviceModel::SortRole);
     sortModel->sort(0);
     // Set the source model then connect to the selection model to get updates
     ui->devicesTV->setModel(sortModel);
     connect(ui->devicesTV->selectionModel(), SIGNAL(selectionChanged(QItemSelection,QItemSelection)),
-            this, SLOT(showProfile()));
-
-    m_model = new DeviceModel(this);
-    connect(m_model, SIGNAL(changed()), this, SLOT(showProfile()));
-    sortModel->setSourceModel(m_model);
+            this, SLOT(showDescription()));
 
+    m_deviceModel = new DeviceModel(this);
+    connect(m_deviceModel, SIGNAL(changed()), this, SLOT(updateSelection()));
+    sortModel->setSourceModel(m_deviceModel);
 
     // Profiles view setup
-    ProfileModel *model = new ProfileModel(this);
-    connect(m_model, SIGNAL(changed()), this, SLOT(showProfile()));
+    m_profileModel = new ProfileModel(this);
+    connect(m_profileModel, SIGNAL(changed()), this, SLOT(updateSelection()));
     // Filter Proxy for the menu
     m_profilesFilter = new QSortFilterProxyModel(this);
-    m_profilesFilter->setSourceModel(model);
+    m_profilesFilter->setSourceModel(m_profileModel);
     m_profilesFilter->setFilterRole(ProfileModel::ColorspaceRole);
-    m_profilesFilter->setSortRole(ProfileModel::ProfileDisplayNameSourceRole);
+    m_profilesFilter->setSortRole(ProfileModel::SortRole);
     m_profilesFilter->setDynamicSortFilter(true);
     m_profilesFilter->sort(0);
 
     // Sort Proxy for the View
     QSortFilterProxyModel *profileSortModel = new QSortFilterProxyModel(this);
-    profileSortModel->setSourceModel(model);
+    profileSortModel->setSourceModel(m_profileModel);
     profileSortModel->setDynamicSortFilter(true);
     profileSortModel->setSortRole(ProfileModel::SortRole);
     profileSortModel->sort(0);
     ui->profilesTV->setModel(profileSortModel);
     connect(ui->profilesTV->selectionModel(), SIGNAL(selectionChanged(QItemSelection,QItemSelection)),
-            this, SLOT(showProfile()));
+            this, SLOT(showDescription()));
     connect(profileSortModel, SIGNAL(dataChanged(QModelIndex,QModelIndex)),
-            this, SLOT(showProfile()));
+            this, SLOT(showDescription()));
+
+
+    // Make sure we know is colord is running
+    QDBusServiceWatcher *watcher;
+    watcher = new QDBusServiceWatcher("org.freedesktop.ColorManager",
+                                      QDBusConnection::systemBus(),
+                                      QDBusServiceWatcher::WatchForOwnerChange,
+                                      this);
+    connect(watcher, SIGNAL(serviceOwnerChanged(QString,QString,QString)),
+            m_deviceModel, SLOT(serviceOwnerChanged(QString,QString,QString)));
+    connect(watcher, SIGNAL(serviceOwnerChanged(QString,QString,QString)),
+            m_profileModel, SLOT(serviceOwnerChanged(QString,QString,QString)));
+    connect(watcher, SIGNAL(serviceOwnerChanged(QString,QString,QString)),
+            ui->profile, SLOT(serviceOwnerChanged(QString,QString,QString)));
 
     // Creates a ColorD interface, it must be created with new
     // otherwise the object will be deleted when this block ends
@@ -154,16 +169,13 @@ ColordKCM::ColordKCM(QWidget *parent, const QVariantList &args) :
     connect(signalMapper, SIGNAL(mapped(int)),
             ui->tabWidget, SLOT(setCurrentIndex(int)));
     connect(signalMapper, SIGNAL(mapped(int)),
-            this, SLOT(showProfile()));
+            this, SLOT(showDescription()));
 
+    // make sure the screen is split on the half
     QList<int> sizes;
     sizes << width() / 2;
     sizes << width() / 2;
     ui->splitter->setSizes(sizes);
-
-    // Make sure we have something selected
-    QTimer::singleShot(0, this, SLOT(adjustTabWidgetSize()));
-    QTimer::singleShot(0, this, SLOT(showProfile()));
 }
 
 ColordKCM::~ColordKCM()
@@ -171,20 +183,25 @@ ColordKCM::~ColordKCM()
     delete ui;
 }
 
-void ColordKCM::adjustTabWidgetSize()
+void ColordKCM::load()
 {
     // Force the profile widget to get a proper height in case
     // the stacked widget is showing the info page first
     if (ui->stackedWidget->currentWidget() != ui->profile_page) {
+        // This is highly needed otherwise the size get wrong on System Settings
         ui->stackedWidget->setCurrentWidget(ui->profile_page);
     }
+    ui->devicesTV->setFocus();
 
     // align the tabbar to the list view
     int offset = ui->profile->innerHeight() - ui->devicesTV->viewport()->height();
     ui->offsetSpacer->changeSize(30, offset, QSizePolicy::Fixed, QSizePolicy::Fixed);
+
+    // Make sure we have something selected
+    showDescription();
 }
 
-void ColordKCM::showProfile()
+void ColordKCM::showDescription()
 {
     QModelIndex index = currentIndex();
     if (!index.isValid()) {
@@ -202,17 +219,8 @@ void ColordKCM::showProfile()
     }
 
     // Check if we can remove the Profile
-    bool enable = false;
-    if (ui->tabWidget->currentIndex() == 1) {
-        QString filename = index.data(ProfileModel::FilenameRole).toString();
-        QFileInfo fileInfo(filename);
-        if (!filename.isNull() && fileInfo.isWritable()) {
-            enable = true;
-        }
-    } else if (index.parent().isValid()) {
-        // It's ok to remove profiles from devices
-        enable = true;
-    }
+    bool enable;
+    enable = index.data(ProfileModel::CanRemoveProfileRole).toBool();
     ui->removeProfileBt->setEnabled(enable);
 }
 
@@ -235,7 +243,7 @@ void ColordKCM::addProfileFile()
     } else if (index.isValid()) {
         // Store the device kind and device object path
         // so that we assign the profile to the device when
-        // ProfileAdded is emited
+        // ProfileAdded is emitted
         QString kind;
         QDBusObjectPath devicePath;
         kind = index.data(DeviceModel::ProfileKindRole).toString();
@@ -252,6 +260,24 @@ void ColordKCM::addProfileAction(QAction *action)
     addProvileToDevice(profileObject, deviceObject);
 }
 
+void ColordKCM::updateSelection()
+{
+    QAbstractItemView *view;
+    if (sender() == m_deviceModel) {
+        view = ui->devicesTV;
+    } else {
+        view = ui->profilesTV;
+    }
+
+    QItemSelection selection;
+    selection = view->selectionModel()->selection();
+    // Make sure we have an index selected
+    if (selection.indexes().isEmpty()) {
+        view->selectionModel()->select(view->model()->index(0, 0),
+                                       QItemSelectionModel::SelectCurrent);
+    }
+}
+
 void ColordKCM::removeProfile()
 {
     QModelIndex index = currentIndex();
@@ -334,18 +360,14 @@ void ColordKCM::fillMenu()
         // Create the profile action
         QAction *action;
         QString title;
-        title = profileIndex.data(ProfileModel::ProfileDisplayNameSourceRole).toString();
+        title = profileIndex.data().toString();
         action = m_addAvailableMenu->addAction(title);
         action->setData(profileIndex.data(ProfileModel::ObjectPathRole));
         action->setProperty(DEVICE_PATH, devicePath);
     }
 
-    if (childCount == 0) {
-        // There are no available profiles
-        m_addAvailableMenu->setEnabled(false);
-        return;
-    }
-    m_addAvailableMenu->setEnabled(m_profilesFilter->rowCount());
+    // If the menu doesn't have any entries it should be disabled
+    m_addAvailableMenu->setEnabled(!m_addAvailableMenu->actions().isEmpty());
 }
 
 void ColordKCM::on_tabWidget_currentChanged(int index)
@@ -431,17 +453,12 @@ QModelIndex ColordKCM::currentIndex() const
     }
 
     QItemSelection selection;
-    // we need to map the selection to source to get the real indexes
     selection = view->selectionModel()->selection();
-    // select the first printer if there are profiles
-    if (selection.indexes().isEmpty()) {
-        view->selectionModel()->select(view->model()->index(0, 0),
-                                       QItemSelectionModel::Select);
-        return ret;
+    // select the first index if the selection is not empty
+    if (!selection.indexes().isEmpty()) {
+        ret = selection.indexes().first();
     }
 
-    ret = selection.indexes().first();
-
     return ret;
 }
 
