@@ -42,17 +42,17 @@
 #include <QtDBus/QDBusMetaType>
 #include <QtDBus/QDBusUnixFileDescriptor>
 #include <QtDBus/QDBusServiceWatcher>
-#include <QDesktopWidget>
 #include <QApplication>
-#include <QDesktopWidget>
 
 K_PLUGIN_FACTORY(ColorDFactory, registerPlugin<ColorD>();)
 K_EXPORT_PLUGIN(ColorDFactory("colord"))
 
+typedef QList<QDBusObjectPath> ObjectPathList;
+
 ColorD::ColorD(QObject *parent, const QVariantList &args) :
     KDEDModule(parent),
-    m_has_1_3(false),
-    m_dirWatch(0)
+    m_dirWatch(0),
+    m_has_1_3(false)
 {
     // There's not much use for args in a KCM
     Q_UNUSED(args)
@@ -60,6 +60,7 @@ ColorD::ColorD(QObject *parent, const QVariantList &args) :
     // Register this first or the first time will fail
     qDBusRegisterMetaType<StringStringMap>();
     qDBusRegisterMetaType<QDBusUnixFileDescriptor>();
+    qDBusRegisterMetaType<ObjectPathList>();
 
     // connect to colord using DBus
     connectToColorD();
@@ -76,7 +77,8 @@ ColorD::ColorD(QObject *parent, const QVariantList &args) :
     connect(watcher, SIGNAL(serviceOwnerChanged(QString,QString,QString)),
             this, SLOT(serviceOwnerChanged(QString,QString,QString)));
 
-    init();
+    // init later on the event loop as this takes quite some time
+    QTimer::singleShot(0, this, SLOT(init()));
 }
 
 ColorD::~ColorD()
@@ -85,11 +87,11 @@ ColorD::~ColorD()
 
 void ColorD::init()
 {
+    // Scan all the *.icc files later on the event loop as this takes quite some time
+    scanHomeDirectory();
+
     // Check outputs add all connected outputs
     checkOutputs();
-
-    // Scan all the *.icc files later on the event loop as this takes quite some time
-    QTimer::singleShot(0, this, SLOT(scanHomeDirectory()));
 }
 
 void ColorD::reset()
@@ -113,7 +115,6 @@ void ColorD::addProfile(const QFileInfo &fileInfo)
     profile.seek(0);
 
     QString profileId = QLatin1String("icc-") + hash;
-    kDebug() << "profileId" << profileId;
 
     bool fdPass;
     fdPass = (QDBusConnection::systemBus().connectionCapabilities() & QDBusConnection::UnixFileDescriptorPassing);
@@ -137,7 +138,66 @@ void ColorD::addProfile(const QFileInfo &fileInfo)
 
     QDBusReply<QDBusObjectPath> reply = QDBusConnection::systemBus().call(message, QDBus::BlockWithGui);
 
-    kDebug() << "created profile" << reply.value().path();
+    kDebug() << "created profile" << profileId << reply.value().path();
+}
+
+void ColorD::addProfileToDevice(const QDBusObjectPath &profilePath, const QDBusObjectPath &devicePath)
+{
+    QDBusMessage message;
+    message = QDBusMessage::createMethodCall(QLatin1String("org.freedesktop.ColorManager"),
+                                             devicePath.path(),
+                                             QLatin1String("org.freedesktop.ColorManager.Device"),
+                                             QLatin1String("AddProfile"));
+    message << QString("soft"); // Relation
+    message << qVariantFromValue(profilePath); // Profile Path
+
+    /* call Device.AddProfile() with the device and profile object paths */
+    QDBusConnection::systemBus().send(message);
+    kDebug() << "Profile added" << devicePath.path() << profilePath.path();
+}
+
+void ColorD::addEdidProfileToDevice(Output &output)
+{
+    // Ask for profiles
+    QDBusMessage message;
+    message = QDBusMessage::createMethodCall(QLatin1String("org.freedesktop.ColorManager"),
+                                             QLatin1String("/org/freedesktop/ColorManager"),
+                                             QLatin1String("org.freedesktop.ColorManager"),
+                                             QLatin1String("GetProfiles"));
+    QDBusReply<ObjectPathList> paths = QDBusConnection::systemBus().call(message, QDBus::BlockWithGui);
+
+    // Search through all profiles to see if the edid md5 matches
+    foreach (const QDBusObjectPath &profilePath, paths.value()) {
+        StringStringMap metadata = getProfileMetadata(profilePath);
+        if (metadata.contains(QLatin1String("EDID_md5"))) {
+            // check if md5 matches
+            if (metadata[QLatin1String("EDID_md5")] == output.edidHash()) {
+                kDebug() << "Foud EDID profile for device" << profilePath.path() << output.path().path();
+                addProfileToDevice(profilePath, output.path());
+            }
+        }
+    }
+}
+
+StringStringMap ColorD::getProfileMetadata(const QDBusObjectPath &profilePath)
+{
+    StringStringMap ret;
+    QDBusMessage message;
+    message = QDBusMessage::createMethodCall(QLatin1String("org.freedesktop.ColorManager"),
+                                             profilePath.path(),
+                                             QLatin1String("org.freedesktop.DBus.Properties"),
+                                             QLatin1String("Get"));
+    message << QString("org.freedesktop.ColorManager.Profile"); // Interface
+    message << QString("Metadata"); // Propertie Name
+    QDBusReply<QVariant> reply = QDBusConnection::systemBus().call(message, QDBus::BlockWithGui);
+    if (!reply.isValid()) {
+        kWarning() << "Failed to get Metadata from profile" << profilePath.path() << reply.error().message();
+        return ret;
+    }
+
+    QDBusArgument argument = reply.value().value<QDBusArgument>();
+    ret = qdbus_cast<StringStringMap>(argument);
+    return ret;
 }
 
 QString ColorD::profilesPath() const
@@ -234,18 +294,34 @@ void ColorD::addOutput(Output &output)
 
     // grabing the device even if edid is not valid
     // if handles the fallback name if it's not valid
-    deviceId = edid.deviceId(output.name());
+    deviceId = output.id();
 
+
+    // EDID profile Creation
+    // Creates a path for EDID generated profile
+    QString autogenPath = profilesPath();
+    QDir profilesDir(autogenPath);
+    if (!profilesDir.exists()) {
+        kWarning() << "Icc path" << profilesDir.path() << "does not exist";
+        if (!profilesDir.mkpath(autogenPath)) {
+            kWarning() << "Failed to create icc path '~/.local/share/icc'";
+        }
+    }
+    autogenPath.append(QLatin1String("edid-") % edid.hash() % QLatin1String(".icc"));
+    ProfileUtils::createIccProfile(isLaptop, edid, autogenPath);
+
+
+    // build up a map with the output properties to send to colord
     StringStringMap properties;
-    properties["Kind"] = "display";
-    properties["Mode"] = "physical";
-    properties["Colorspace"] = "rgb";
+    properties["Kind"] = QLatin1String("display");
+    properties["Mode"] = QLatin1String("physical");
+    properties["Colorspace"] = QLatin1String("rgb");
     properties["Vendor"] = edidVendor;
     properties["Model"] = edidModel;
     properties["Serial"] = edidSerial;
     properties["XRANDR_name"] = output.name();
 
-    /* call CreateDevice() with a device_id  */
+    // call CreateDevice() with a device_id
     QDBusMessage message;
     message = QDBusMessage::createMethodCall(QLatin1String("org.freedesktop.ColorManager"),
                                              QLatin1String("/org/freedesktop/ColorManager"),
@@ -255,33 +331,19 @@ void ColorD::addOutput(Output &output)
     // We use temp because if we crash or quit the device gets removedf
     message << qVariantFromValue(QString("temp"));
     message << qVariantFromValue(properties);
-    kWarning() << "Adding device id:" << deviceId;
+    kDebug() << "Adding device id:" << deviceId;
+    kDebug() << "Output Hash:" << output.edidHash();
     QDBusReply<QDBusObjectPath> reply = QDBusConnection::systemBus().call(message, QDBus::BlockWithGui);
     if (reply.isValid()) {
-        kDebug() << "created colord device" << reply.value().path();
-
+        kDebug() << "Created colord device" << reply.value().path();
         // Store the output path into our Output class
         output.setPath(reply.value());
 
         // Store the connected output into the connected list
         m_connectedOutputs << output;
 
-        // Creates a path for out auto generated profile
-        QString autogenPath = profilesPath();
-        QDir profilesDir(autogenPath);
-        if (!profilesDir.exists()) {
-            kWarning() << "Icc path" << profilesDir.path() << "does not exist";
-            if (!profilesDir.mkpath(autogenPath)) {
-                kWarning() << "Failed to create icc path '~/.local/share/icc'";
-            }
-        }
-        autogenPath.append(QLatin1String("edid-") % edid.hash() % QLatin1String(".icc"));
-
-        bool ret;
-        ret = ProfileUtils::createIccProfile(isLaptop, edid, autogenPath);
-        if (ret == false) {
-            kWarning() << "Failed to create auto profile";
-        }
+        // Check if there is any EDID profile to be added
+        addEdidProfileToDevice(output);
     } else {
         kWarning() << "Failed to register device:" << reply.error().message();
     }
@@ -520,65 +582,48 @@ void ColorD::checkOutputs()
     }
 }
 
-void ColorD::profileAdded(const QDBusObjectPath &objectPath)
+void ColorD::profileAdded(const QDBusObjectPath &profilePath)
 {
-    /* check if the EDID_md5 Profile.Metadata matches any connected
-     * XRandR devices (e.g. lvds1), otherwise ignore */
-    QDBusMessage message;
-    message = QDBusMessage::createMethodCall(QLatin1String("org.freedesktop.ColorManager"),
-                                             objectPath.path(),
-                                             QLatin1String("org.freedesktop.DBus.Properties"),
-                                             QLatin1String("Get"));
-    message << QString("org.freedesktop.ColorManager.Profile"); // Interface
-    message << QString("Metadata"); // Propertie Name
-    QDBusReply<QVariant> reply = QDBusConnection::systemBus().call(message, QDBus::BlockWithGui);
-    if (!reply.isValid()) {
-        kWarning() << "Failed to get Metadata from profile" << objectPath.path() << reply.error().message();
-        return;
-    }
-    QDBusArgument argument = reply.value().value<QDBusArgument>();
-    StringStringMap metadata = qdbus_cast<StringStringMap>(argument);
-    kDebug() << reply.value() << metadata;
-    kDebug() << metadata.size();
-
-    StringStringMap::const_iterator i = metadata.constBegin();
-    while (i != metadata.constEnd()) {
-        kDebug() << i.key() << ": " << i.value();
-        if (i.key() == QLatin1String("EDID_md5")) {
-            QString edidHash = i.value();
-            const Output *output = 0;
-            // Get the Crtc of this output
-            foreach (const Output &out, m_connectedOutputs) {
-                if (out.edidHash() == edidHash) {
-                    output = &out;
-                    break;
-                }
-            }
-
-            if (output) {
-                // Found an EDID that matches the md5
-                QDBusMessage message;
-                message = QDBusMessage::createMethodCall(QLatin1String("org.freedesktop.ColorManager"),
-                                                         output->path().path(),
-                                                         QLatin1String("org.freedesktop.ColorManager.Device"),
-                                                         QLatin1String("AddProfile"));
-                message << QString("soft"); // Relation
-                message << qVariantFromValue(objectPath); // Profile Path
-
-                /* call Device.AddProfile() with the device and profile object paths */
-                QDBusConnection::systemBus().send(message);
-                kDebug() << "Profile added" << output->path().path() << objectPath.path();
+    // check if the EDID_md5 Profile.Metadata matches any connected
+    // XRandR devices (e.g. lvds1), otherwise ignore
+    StringStringMap metadata = getProfileMetadata(profilePath);
+    if (metadata.contains(QLatin1String("EDID_md5"))) {
+        QString edidHash = metadata[QLatin1String("EDID_md5")];
+        const Output *output = 0;
+        // Get the Crtc of this output
+        foreach (const Output &out, m_connectedOutputs) {
+            if (out.edidHash() == edidHash) {
+                output = &out;
+                break;
             }
         }
-        ++i;
+
+        if (output) {
+            // Found an EDID that matches the md5
+            addProfileToDevice(profilePath, output->path());
+        }
     }
 }
 
 void ColorD::deviceAdded(const QDBusObjectPath &objectPath)
 {
     kDebug() << "Device added" << objectPath.path();
+//    QDBusInterface deviceInterface(QLatin1String("org.freedesktop.ColorManager"),
+//                                   objectPath.path(),
+//                                   QLatin1String("org.freedesktop.ColorManager.Device"),
+//                                   QDBusConnection::systemBus(),
+//                                   this);
+//    if (!deviceInterface.isValid()) {
+//        return;
+//    }
 
-    /* show a notification that the user should calibrate the device */
+//    // check Device.Kind is "display"
+//    if (deviceInterface.property("Kind").toString() != QLatin1String("display")) {
+//        // not a display device, ignoring
+//        return;
+//    }
+
+    /* show a notification if the user should calibrate the device */
     //TODO
 }
 
@@ -604,14 +649,17 @@ void ColorD::deviceChanged(const QDBusObjectPath &objectPath)
 
 void ColorD::addProfile(const QString &filename)
 {
-    kDebug() << filename;
-    QFileInfo fileInfo(filename);
-    addProfile(fileInfo);
+    // if the changed file is an ICC profile
+    KMimeType::Ptr mimeType;
+    mimeType = KMimeType::findByFileContent(filename);
+    if (mimeType->is(QLatin1String("application/vnd.iccprofile"))) {
+        QFileInfo fileInfo(filename);
+        addProfile(fileInfo);
+    }
 }
 
 void ColorD::removeProfile(const QString &filename)
 {
-    kDebug() << filename;
     QDBusMessage message;
     message = QDBusMessage::createMethodCall(QLatin1String("org.freedesktop.ColorManager"),
                                              QLatin1String("/org/freedesktop/ColorManager"),
